import java.awt.*;
import java.util.Arrays;

/**
   最短路問題に対するダイクストラ法のプログラム例
   http://web1.kcg.edu/~t_ibaraki/shokodo/dijkstra.c
   を Java に移植した。
*/
@SuppressWarnings("serial")
public class Dijkstra extends GraphApplication {
    public Dijkstra(String title, String file) {
        super(title, file);
    }

    /** 丸め誤差の許容値 */
    static final double ZERO = 0.0001;
    /** 現在の最小木を構成する枝の数 */
    private int k = 0;
    /** 最小木 */
    private Edge tree[];
    /** すべての枝 */
    private Edge edges[];

    /** 枝 */
    private class Edge implements Comparable<Edge> {
        float d;            /* 枝長 */
        int end1, end2;     /* 両端点 */
        Edge(int end1, int end2, float d) {
            this.end1 = end1;
            this.end2 = end2;
            this.d = d;
        }
        public int compareTo(Edge other) {
            double diff = d - other.d;
            if (diff > 0) return 1;
            if (diff < 0) return -1;
            int ret = end1 - other.end1;
            if (ret == 0) ret = end2 - other.end2;
            return ret;
        }
        public String toString() {
            return (end1+1) + " " + (end2+1);
        }
    }

    private class Value {         /* 構造体valueの定義 */
        float d;
        int node;
    }

    private class Cell {          /* 構造体cellの定義 */
        int adj;
        int edge;
        Cell next;
    }

    public void drawAllEdges( Graphics g ) {
        if ( edges == null ) return;
        g.setColor( Color.blue );
        for (int i=0; i<edges.length; i++) {
            drawEdge(edges[i].end1, edges[i].end2, g);
        }
        g.setColor( GraphPanel.DEFAULT_EDGE_COLOR );
        for (int i=0; i<k; i++) {
            drawEdge(tree[i].end1, tree[i].end2, g);
        }
    }


    /* ヒープAにおける構造体A[i]とA[j]の交換 */
    private void swap(int i, int j, Value a[], int loc[]) {
        Value tmp = a[i]; a[i] = a[j]; a[j] = tmp;
        loc[a[i].node] = i; loc[a[j].node] = j;           /* locの更新 */
    }


    /* ヒープa[0],...,a[n-1]から最小要素a[0]の出力と除去; n=n-1 */
    private Value deleteMin(Value a[], int loc[], int n) {
        swap(0, n-1, a, loc);
        if (n>1) downMin(0, a, loc, n-1);
        return a[n-1];
    }


    /* ヒープa[0],...,a[n-1]へ新しい要素xの挿入; n=n+1 */
    private void insert(float d, int node, Value a[], int loc[], int n) {
        a[n].d = d;
        a[n].node = node;
        loc[a[n].node] = n;
        upMin(n, a, loc, n+1);
    }


    /* a[i]から下方へ、ヒープ条件回復のためのswap操作を適用 */
    private void downMin(int i, Value a[], int loc[], int n) {
        if (i<0 || i>=n) { 
            throw new IllegalArgumentException("i = " + i + " for n = "+ n);
        }
        int j = 2*i + 1;
        if (j>=n) return;
        if (j+1<n && a[j].d > a[j+1].d+ZERO) j++;
        if (a[j].d < a[i].d-ZERO) {
            swap(i, j, a, loc);
            downMin(j, a, loc, n);
        }
    }


    /* a[i]から上方へ、ヒープ条件回復のためswap操作を適用 */
    private void upMin(int i, Value a[], int loc[], int n) {
        if (i<0 || i>=n) {
            throw new IllegalArgumentException("i = " + i + " for n = "+ n);
        }
        if (i==0) return;
        int j = (i+1)/2-1; 
        if (a[j].d>a[i].d) {
            swap(i, j, a, loc); 
            upMin(j, a, loc, n);
        }
    }


    /**
       節点数n, 枝数m, 枝集合Eをもつグラフの始点 source からの
       最短路木を求めるダイクストラのアルゴリズム。
       結果を tree（最短路木）とdstar（最短路長）に入れる。
    */
    public void dijkstra(int source, Edge E[], float dstar[], int n, int m) {
        System.out.println("shortest path from " + (source+1));
        boolean visited[] = new boolean[n];
        int loc[] = new int[n];
        int edge[] = new int[n];
        Cell adjlist[] = new Cell[n];

        Value heap[] = new Value[n-1];
        for (int i=0; i<heap.length; i++) heap[i] = new Value();

        for (int i = 0; i<n; i++) visited[i] = false;
        visited[source] = true;
        for (int i=0; i<n; i++) { adjlist[i] = null; loc[i] = -1; }
        for (int i=0; i<m; i++) {
            int v1 = E[i].end1;
            int v2 = E[i].end2;    
            Cell r = new Cell();
            r.adj = v2; r.edge = i; r.next = adjlist[v1]; adjlist[v1] = r;
            Cell q = new Cell();
            q.adj = v1; q.edge = i; q.next = adjlist[v2]; adjlist[v2] = q;
        }

        int nh = 0;
        int u = source;
        float du = 0;
        dstar[u] = 0;
        for (k=0; k<n-1; k++) {
            if ( k>0 && !sleep() ) return;
            Cell p = adjlist[u];
            while (p != null) {
                int vadj = p.adj;
                if (!visited[vadj]) {
                    if (loc[vadj] == -1) { /* vadjをヒープに入れる */
                        edge[vadj] = p.edge;
                        insert(du+E[p.edge].d, vadj, heap, loc, nh++);
                    } else { /* すでにヒープにあるvadjの更新 */
                        int i = loc[vadj];
                        if (heap[i].d > du+E[p.edge].d+ZERO) {
                            edge[vadj] = p.edge;
                            heap[i].d = du+E[p.edge].d;
                            upMin(i, heap, loc, nh);
                        }
                    }
                }
                p = p.next;
            }
            Value vmin = deleteMin(heap, loc, nh--);
            u = vmin.node;
            du = vmin.d;  // コメントにすれば Prime 法になる
            dstar[u] = du;
            visited[u] = true;
            tree[k] = E[edge[u]];
            drawEdge(tree[k].end1, tree[k].end2);
            System.out.println(tree[k] + " " + (u+1) + " " + du);
        }
        sleepForever();
    }
    

    /** 区間 [a,b] と 区間 [c,d] が重なるか */
    private boolean intersect(int a, int b, int c, int d) {
        if (a > b) { int swap = a;  a = b;  b = swap; }
        if (c > d) { int swap = c;  c = d;  d = swap; }
        return (a < d || c < b);
    }

    /** 点 p が有向線分 a,b の左右どちらにあるか  */
    private int side(int a, int b, int p) {
        int n  =
            x[p] * (y[a] - y[b]) +
            x[a] * (y[b] - y[p]) +
            x[b] * (y[p] - y[a]);
	if      ( n > 0 ) return  1; // 右
	else if ( n < 0 ) return -1; // 左
	else              return  0; // 線上        
    }

    /** 有向線分 a, b から見て点 c と点 d が両側にあるか */
    private boolean straddle(int a, int b, int c, int d) {
        return (side(a, b, c) * side(a, b, d) < 0);
    }

    /** 枝 e と f が交差するか */
    // http://www.hidecnet.ne.jp/~sinzan/tips/c/c_tip15.htm
    private boolean intersect(Edge e, Edge f) {
        if (!intersect(x[e.end1], x[e.end2], x[f.end1], x[f.end2]) ||
            !intersect(y[e.end1], y[e.end2], y[f.end1], y[f.end2]))
            return false;
        return (straddle(e.end1, e.end2, f.end1, f.end2) &&
                straddle(f.end1, f.end2, e.end1, e.end2));
    }


    public void run() {
        super.run();
        
        int m = n * (n-1) / 2;
        tree = new Edge[n-1];
        float dstar[] = new float[n];    /* 節点への最短路長 */

        // 頂点座標を元に完全グラフを作る
        Edge[] e = new Edge[m];
        int index = 0;
        for (int i=0; i<n; i++) {
            for (int j=0; j<i; j++) {
                e[index++] = new Edge(i, j, d[i][j]);
            }
        }
        Arrays.sort(e);  // 枝の短いもの順にソート

        // 平面上で交わる枝は長い方を取り除く
        index = 0;
        for (int i=0; i<m; i++) {
            boolean intersected = false;
            for (int j=0; j<index; j++) {
                if (intersect(e[i], e[j])) {
                    intersected = true;
                    break;
                }
            }
            if (!intersected) {
                drawEdge(e[i].end1, e[i].end2);
                e[index++] = e[i];
                sleep(1);
            }
        }

        // edges に代入しなおし
        edges = new Edge[index];
        for (int i=0; i<index; i++) edges[i] = e[i];
        System.out.println("network");
        for (int i=0; i<index; i++) {
            System.out.println(edges[i] + " " +edges[i].d);
        }
        // e を捨てる
        for (int i=0; i<e.length; i++) e[i] = null;
        e = null;
        System.gc();
        setReady(true);

        while (thread != null) {
            k = 0;  repaint();
            int source = getStartingIndex();
            dijkstra(source, edges, dstar, n, edges.length);  // 最短路の計算
        }
    }
}
